(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

let get t x = try Some (Hashtbl.find t x) with Not_found -> None

module type QUEUE = sig
  type 'a t
  val empty: unit -> 'a t
  val singleton: 'a -> 'a t
  val push: 'a t -> 'a -> unit
  val pop: 'a t -> 'a option
end

module IntSet = Sets.NarithInlined
type int_set = IntSet.t

module Set = struct
  type 'a t = 'a list
  let empty = []
  let single x = [x]
  let union =
    let rec union h k = 
      match h,k with
	| [],l | l,[] -> l
	| x::h', y::k' -> match compare x y with
	    | 0 -> x::union h' k'
	    | 1 -> x::union h' k
	    | _ -> y::union h k'
    in union
  let add x = union [x]
  let fold = List.fold_right
  let map f = List.fold_left (fun x i -> union [f i] x) []
  let int_map f = List.fold_left (fun x i -> IntSet.add (f i) x) IntSet.empty
end
type 'a set = 'a Set.t

type var = char
type key = char
type gstring_ = (var * (key*bool) list) list
type gstring = (key*bool) list * gstring_
let print_gstring f (alpha,l) =
  let rec atom = function
    | [] -> ()
    | (a,true)::q -> atom q; Format.fprintf f "%c" a
    | (a,false)::q -> atom q; Format.fprintf f "!%c" a
  in
  let rec list = function
    | [] -> ()
    | (p,alpha)::q -> list q; atom alpha; Format.fprintf f "%c" p
  in list l; atom alpha

module Span = struct
  type 's t = (var*'s) list * 's
  let empty z = [],z
  let single a x z = [a,x],z
  let merge f =
    let rec merge h k = match h,k with
      | [],l | l,[] -> l
      | (a,x as ax)::h', (b,y as ay)::k' -> 
	match compare a b with
	  | 0 -> (a, f x y)::merge h' k'
	  | 1 -> ax::merge h' k
	  | _ -> ay::merge h k'
    in fun (h,z) (k,z') -> assert(z==z'); merge h k,z
  let map f (h,z) = List.map (fun (a,x) -> (a, f x)) h, f z
  let iter2 f w (h,z) (k,z') =
    assert(z==z');
    let rec iter2 h k = match h,k with
      | [],[] -> ()
      | [],(a,y)::k -> f a w z y; iter2 [] k
      | (a,x)::h,[] -> f a w x z; iter2 h []
      | (a,x)::h', (b,y)::k' -> 
	match compare a b with
	  | 0 -> f a w x y; iter2 h' k'
	  | 1 -> f a w x z; iter2 h' k
	  | _ -> f b w z y; iter2 h  k'
    in iter2 h k
end
type 's span = 's Span.t
