(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

(** Common definitions and utilities *)

(** wrapper around [Hashtbl.find] *)
val get : ('a, 'b) Hashtbl.t -> 'a -> 'b option

(** signature for queues *)
module type QUEUE = sig
  type 'a t
  val empty : unit -> 'a t
  val singleton : 'a -> 'a t
  val push : 'a t -> 'a -> unit
  val pop : 'a t -> 'a option
end

(** efficient implementation of sets of integers *)
module IntSet: Sets.T
type int_set = IntSet.t

(** (not so efficient) implementation of polymorphic sets *)
module Set: sig
  type 'a t
  val empty : 'a t
  val single : 'a -> 'a t
  val union : 'a t -> 'a t -> 'a t
  val add : 'a -> 'a t -> 'a t
  val map : ('a -> 'b) -> 'a t -> 'b t
  val fold : ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
  val int_map : ('a -> int) -> 'a t -> int_set
  end
type 'a set = 'a Set.t

(** the type of KAT atomic Kleene elements *)
type var = char

(** the type of KAT atomic tests *)
type key = char

(** unterminated guarded strings (for counter-examples) *)
type gstring_ = (var * (key*bool) list) list
(** guarded strings *)
type gstring = (key*bool) list * gstring_

(** pretty-printing for guarded strings *)
val print_gstring: Format.formatter -> gstring -> unit


(** spans: sparse polymorphic maps indexed by [var] 
    these maps are used to represent explicit part of the
    transitions of guarded string automata *)
module Span: sig
  type 'a t
  (** [empty z] is the constant to [z] map *)
  val empty : 'a -> 'a t
  (** [single p x z] is map whose value is [x] on [p] and [z] everywhere else *)
  val single : var -> 'a -> 'a -> 'a t
  (** [merge f m n] applies [f] pointwise to [m] and [n], 
      assume that [m] and [z] share the same default value [z], 
      and that [f z z = z] *)
  val merge : ('a -> 'a -> 'a) -> 'a t -> 'a t -> 'a t
  (** pointwise application of a function  *)
  val map : ('a -> 'b) -> 'a t -> 'b t
  (** [iter2 f m n] applies [f] to all non-trivial pairs obtained by zipping [m] and [n] *)
  val iter2 : (var -> 'gs -> 'a -> 'a -> unit) -> 'gs -> 'a t -> 'a t -> unit
end
type 's span = 's Span.t
